﻿#!/bin/sh
# cd /usr/lib/hildon-desktop
# python2.5 /usr/lib/hildon-desktop/bluezwitch.py

APP_TITLE = "Bluezwitch"
APP_VERSION = "0.2"

import gobject
import gtk
import hildondesktop
import hildon
import os
import cairo
import osso
import string
import time 
import gconf

import dbus

APP_NAME = APP_TITLE.lower()
LOGO = '/opt/tomch.com/bluezwitch/%s.png' % APP_NAME
GCONFPATH = "/apps/Maemo/tomch.com/%s" % APP_NAME


class BluezwitchHomePlugin(hildondesktop.HomePluginItem):
  defaultMSGOFF = "Bluetooth set off"
    
  def __init__(self):
    hildondesktop.HomePluginItem.__init__(self)
    self.set_settings(True)
    self.connect("show-settings", self.show_options)

    self.isStopped = True 
    self.update_info()
    self.transparency = 0.0

    #Load the settings using GConf
    conf_client = gconf.client_get_default()
    conf_client.add_dir(GCONFPATH, gconf.CLIENT_PRELOAD_NONE)
    self.MSGOFF=conf_client.get_string(GCONFPATH+"/msgoff")

    #Fallback to default on first launch
    if not self.MSGOFF:
      self.MSGOFF=self.defaultMSGOFF
      
    self.bus = dbus.SystemBus();

    self.root = self.bus.get_object('org.bluez', '/')
    self.manager = dbus.Interface(self.root, 'org.bluez.Manager')
    self.defaultAdapter = self.manager.DefaultAdapter()
    print "Default adapter is at", self.defaultAdapter

    self.obj = self.bus.get_object('org.bluez', self.defaultAdapter)
    self.adapter = dbus.Interface(self.obj, 'org.bluez.Adapter')
    

  def update_info(self, widget=None):

    self.osso_context = osso.Context("com.tomch.bluezwitch", str(APP_VERSION), False)
    self.osso_rpc = osso.Rpc(self.osso_context)
   
    self.hbox = gtk.HBox()
    self.vbox = gtk.VBox()
    self.e_eventbox = gtk.EventBox()
    self.e_eventbox.set_size_request(64, 72)           
    self.e_eventbox.set_events(gtk.gdk.BUTTON_PRESS_MASK)
    self.e_eventbox.connect("button-release-event", self.on_toggle)
    self.hbox.pack_start(self.e_eventbox, True, True, 10)
    self.hbox.show_all()
    self.add(self.hbox)
    
  def do_expose_event(self, event):
    cr = self.window.cairo_create()
    cr.set_source_rgba(1.0, 1.0, 1.0, self.transparency) # Transparent
    cr.set_operator(cairo.OPERATOR_SOURCE)
    cr.paint()   

    # Paint icon for eventbox
    self.draw_e("bluezwitch.png")

    # draw rounded rect
    width, height = self.allocation[2], self.allocation[3]
    
    x0 = 0
    y0 = 0
    radius = min(15, width/2, height/2)  #/*< and an approximate curvature radius */
    x1 = x0 + width
    y1 = y0 + height 
    
    cr.move_to  (x0, y0 + radius)
    cr.arc (x0 + radius, y0 + radius, radius, 3.14, 1.5 * 3.14)
    cr.line_to (x1 - radius, y0)
    cr.arc (x1 - radius, y0 + radius, radius, 1.5 * 3.14, 0.0)
    cr.line_to (x1 , y1 - radius)
    cr.arc (x1 - radius, y1 - radius, radius, 0.0, 0.5 * 3.14)
    cr.line_to (x0 + radius, y1)
    cr.arc (x0 + radius, y1 - radius, radius, 0.5 * 3.14, 3.14)
    cr.close_path ()

    cr.set_source_rgba (0.0, 0.0, 0.0, self.transparency)
    cr.fill_preserve ()
    return False

  def draw_e(self,icon):
    cr_e = self.e_eventbox.window.cairo_create()
    cr_e.set_source_rgba(0.0, 0.0, 0.0, self.transparency) # Transparent
    cr_e.set_operator(cairo.OPERATOR_SOURCE)
    cr_e.paint()
    
    cr_e.set_operator(cairo.OPERATOR_OVER)
    image = cairo.ImageSurface.create_from_png("/opt/tomch.com/bluezwitch/" + icon)
    cr_e.set_source_surface(image, 0, 7)
    cr_e.paint()
    self.cr_e = cr_e
    while gtk.events_pending():
      gtk.main_iteration()
          
  def do_realize(self):
    screen = self.get_screen()
    colormap = screen.get_rgba_colormap()
    self.set_colormap(colormap)
    hildondesktop.HomePluginItem.do_realize(self) 

  def on_toggle(self, button, data=None):
  
    self.draw_e("bluezwitch-grey.png")     
    props = self.adapter.GetProperties()
    powered = props['Powered']
    if powered:
      self.adapter.SetProperty('Powered',False) 
      banner = hildon.hildon_banner_show_information(self, "", self.MSGOFF)
      time.sleep(0.5) 
    else:
      self.adapter.SetProperty('Powered',True)   
      time.sleep(0.5) 
    self.draw_e("bluezwitch.png")

  
  #Setup the main options/settings dialogue (the one accessible through the tool icon when in Hildon Desktop Edit mode)
  def show_options(self, widget):
      dialog = gtk.Dialog("%s Options" % APP_TITLE, None, gtk.DIALOG_DESTROY_WITH_PARENT)

      settings_button = hildon.Button(gtk.HILDON_SIZE_HALFSCREEN_WIDTH | gtk.HILDON_SIZE_FINGER_HEIGHT, hildon.BUTTON_ARRANGEMENT_VERTICAL)
      settings_button.set_text("Settings", "Change how %s looks and works" % APP_TITLE)
      settings_button.set_alignment(0,0,0,0)
      settings_button.connect('clicked', self.show_settings)
              
      help_button = hildon.Button(gtk.HILDON_SIZE_HALFSCREEN_WIDTH | gtk.HILDON_SIZE_FINGER_HEIGHT, hildon.BUTTON_ARRANGEMENT_VERTICAL)
      help_button.set_text("Help", "Get help on how to use %s" % APP_TITLE)
      help_button.set_alignment(0,0,0,0)
      help_button.connect('clicked', self.open_help) 
      
      about_button = hildon.Button(gtk.HILDON_SIZE_HALFSCREEN_WIDTH | gtk.HILDON_SIZE_FINGER_HEIGHT, hildon.BUTTON_ARRANGEMENT_VERTICAL)
      about_button.set_text("About", "More about Author, Copyright and License")
      about_button.set_alignment(0,0,0,0)
      about_button.connect("clicked", self.show_about)

      donate_button = hildon.Button(gtk.HILDON_SIZE_HALFSCREEN_WIDTH | gtk.HILDON_SIZE_FINGER_HEIGHT, hildon.BUTTON_ARRANGEMENT_VERTICAL)
      donate_button.set_text("Donate", "Like the software? Support the effort :-)")
      donate_button.set_alignment(0,0,0,0)
      donate_button.connect('clicked', self.open_donate)

      line1 = gtk.HBox()
      line2 = gtk.HBox()        
      line1.pack_start(settings_button, True, True, 0)
      line1.pack_start(help_button, True, True, 0)
      line2.pack_start(about_button, True, True, 0)
      line2.pack_start(donate_button, True, True, 0)
      dialog.vbox.pack_start(line1, True, True, 0)  
      dialog.vbox.pack_start(line2, True, True, 0)  
       
      dialog.show_all()
      dialog.run()
      dialog.destroy()
  
  #Link to a help page on the maemo wiki    
  def open_help(self, widget): 
      self.osso_rpc.rpc_run_with_defaults("osso_browser", "open_new_window", ("http://wiki.maemo.org/"+APP_TITLE,))

  #Link to a donation page using PayPal    
  def open_donate(self, widget):
      self.osso_rpc.rpc_run_with_defaults("osso_browser", "open_new_window", ("https://www.paypal.com/cgi-bin/webscr?cmd=_s-xclick&hosted_button_id=NJ4HVUZKB2CZY",))                
  
  #Setup the settings UI , store changed parameters  
  def show_settings(self, widget):
      dialog = gtk.Dialog("Settings", None, gtk.DIALOG_DESTROY_WITH_PARENT | gtk.DIALOG_NO_SEPARATOR)
      search_button = dialog.add_button(gtk.STOCK_SAVE, gtk.RESPONSE_OK)
      
      main_vbox = gtk.VBox()
      
      ts = hildon.TouchSelector()
      ts.add(main_vbox)
      
      hboxPresets = gtk.HBox()
      presetsInput = hildon.Entry(gtk.HILDON_SIZE_AUTO)
      presetsInput.set_text(self.MSGOFF)
      hboxPresets.pack_start(gtk.Label("BT Off message"), False, False, 0)
      hboxPresets.pack_start(presetsInput)

      main_vbox.pack_start(hboxPresets)

      dialog.vbox.add(ts)
      dialog.show_all()
      response = dialog.run()
             
      #Parse the updated settings        
      if response == gtk.RESPONSE_OK:
         try:
           self.MSGOFF = presetsInput.get_text()      
         except Exception:
           self.MSGOFF = self.defaultMSGOFF
           self.log('Invalid message. Reset to default value.')
         
         #Store the updated settings
         conf_client = gconf.client_get_default()
         conf_client.add_dir(GCONFPATH, gconf.CLIENT_PRELOAD_NONE)
         try:
           conf_client.set_string(GCONFPATH+"/msgoff", self.MSGOFF)
         except Exception:
           self.log('Unable to save settings in GConf.')
      
      dialog.destroy()
  
  #Show the about dialog    
  def show_about(self, widget):
      dialog = gtk.AboutDialog()
      dialog.set_title("About")
      dialog.set_name(APP_TITLE)
      dialog.set_version(APP_VERSION)
      dialog.set_copyright("Copyright 2010 Tom Waelti")
      dialog.set_authors(["Tom Waelti <twaelti@gmail.com>",""])
      dialog.set_logo(gtk.gdk.pixbuf_new_from_file(LOGO))
      dialog.set_comments("All logos and trademarks are property of their respective owners and are used for informational purposes only.")
      #dialog.set_license("GPL 3")
      dialog.set_license("""This program is free software: you can redistribute it and/or
modify it under the terms of the GNU General Public License as 
published by the Free Software Foundation, either version 3 of
the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied 
warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
PURPOSE.  See the GNU General Public License for more details.

You should have received a copy of the GNU General Public 
License along with this program.  If not, see
<http://www.gnu.org/licenses/>.""")
      dialog.set_wrap_license(True)
      dialog.show_all()
      dialog.run()
      dialog.destroy() 
  
hd_plugin_type = BluezwitchHomePlugin

if __name__ == "__main__":
    gobject.type_register(hd_plugin_type)
    obj = gobject.new(hd_plugin_type, plugin_id="plugin_id")
    obj.show_all()
    gtk.main()
